@@ -97,9 +97,9 @@ class ExplicitCurve:
 
 ################ 多项式曲线 ################
 
-class BasicPolynomial(ExplicitCurve):
+class BasePolynomial(ExplicitCurve):
     def __init__(self, coef=None, valid_x_range=None):
-        super(BasicPolynomial, self).__init__()
+        super(BasePolynomial, self).__init__()
 
         self._derivative_coefs = {}
 
@@ -175,7 +175,7 @@ class BasicPolynomial(ExplicitCurve):
 
 
     def calc_point(self, x):
-        x = np.array(x)
+        x = np.array(x, dtype=np.float)
         extra_mask = self._out_of_range_flag(x)
         val = np.empty_like(x)
         val[extra_mask] = self.extrapolate(x[extra_mask], order=0) # 数据范围外，按外推规则计算
@@ -183,7 +183,7 @@ class BasicPolynomial(ExplicitCurve):
         return val
 
     def calc_first_derivative(self,x):
-        x = np.array(x)
+        x = np.array(x, dtype=np.float)
         extra_mask = self._out_of_range_flag(x)
         val = np.empty_like(x)
         val[extra_mask] = self.extrapolate(x[extra_mask], order=1)  # 数据范围外，按外推规则计算
@@ -191,7 +191,7 @@ class BasicPolynomial(ExplicitCurve):
         return val
 
     def calc_second_derivative(self,x):
-        x = np.array(x)
+        x = np.array(x, dtype=np.float)
         extra_mask = self._out_of_range_flag(x)
         val = np.empty_like(x)
         val[extra_mask] = self.extrapolate(x[extra_mask], order=2)  # 数据范围外，按外推规则计算
@@ -199,7 +199,7 @@ class BasicPolynomial(ExplicitCurve):
         return val
 
     def calc_third_derivative(self, x):
-        x = np.array(x)
+        x = np.array(x, dtype=np.float)
         extra_mask = self._out_of_range_flag(x)
         val = np.empty_like(x)
         val[extra_mask] = self.extrapolate(x[extra_mask], order=3)  # 数据范围外，按外推规则计算
@@ -215,7 +215,7 @@ class BasicPolynomial(ExplicitCurve):
         self.set_coef(coef)
 
 
-class CubicPolynomial(BasicPolynomial):
+class CubicPolynomial(BasePolynomial):
     def __init__(self, coef=None, valid_x_range=None):
         super(CubicPolynomial, self).__init__(coef, valid_x_range)
 
@@ -260,7 +260,7 @@ class CubicPolynomial(BasicPolynomial):
         cp.two_point_boundary_value(x0, y0, math.tan(yaw0), xe, ye, math.tan(yawe))
         return cp
 
-class QuarticPolynomial(BasicPolynomial):
+class QuarticPolynomial(BasePolynomial):
     def __init__(self, coef=None, valid_x_range=None):
         super(QuarticPolynomial, self).__init__(coef, valid_x_range)
         # self.t_range:list = []
@@ -347,7 +347,7 @@ class QuarticPolynomial(BasicPolynomial):
     #
     #     return xt
 
-class QuinticPolynomial(BasicPolynomial):
+class QuinticPolynomial(BasePolynomial):
 
     def __init__(self, coef=None, valid_x_range=None):
         super(QuinticPolynomial, self).__init__(coef, valid_x_range)
@@ -498,11 +498,6 @@ class PiecewiseQuinticPolynomial(ExplicitCurve):
 
 
 
-
-
-
-
-
 ################ 插值曲线 ################
 class InterpolationCurve(ExplicitCurve):
     def __init__(self, x=None, y=None):
@@ -541,7 +536,7 @@ class InterpolationCurve(ExplicitCurve):
         pass
 
     def calc_point(self, x):
-        x = np.array(x)
+        x = np.array(x, dtype=np.float)
         extra_mask = self._out_of_range_flag(x)
         val = np.empty_like(x)
         val[extra_mask] = self.extrapolate(x[extra_mask], order=0)  # 数据范围外，按外推规则计算
@@ -549,7 +544,7 @@ class InterpolationCurve(ExplicitCurve):
         return val
 
     def calc_first_derivative(self, x):
-        x = np.array(x)
+        x = np.array(x, dtype=np.float)
         extra_mask = self._out_of_range_flag(x)
         val = np.empty_like(x)
         val[extra_mask] = self.extrapolate(x[extra_mask], order=1)  # 数据范围外，按外推规则计算
@@ -557,7 +552,7 @@ class InterpolationCurve(ExplicitCurve):
         return val
 
     def calc_second_derivative(self, x):
-        x = np.array(x)
+        x = np.array(x, dtype=np.float)
         extra_mask = self._out_of_range_flag(x)
         val = np.empty_like(x)
         val[extra_mask] = self.extrapolate(x[extra_mask], order=2)  # 数据范围外，按外推规则计算
@@ -565,7 +560,7 @@ class InterpolationCurve(ExplicitCurve):
         return val
 
     def calc_third_derivative(self, x):
-        x = np.array(x)
+        x = np.array(x, dtype=np.float)
         extra_mask = self._out_of_range_flag(x)
         val = np.empty_like(x)
         val[extra_mask] = self.extrapolate(x[extra_mask], order=3)  # 数据范围外，按外推规则计算
@@ -575,6 +570,52 @@ class InterpolationCurve(ExplicitCurve):
 
 
 class CubicSpline(InterpolationCurve):
+    """
+    CubicSpline class
+    三次样条插值
+    """
+    def __init__(self, x=None, y=None):
+        self._sp_csp = None # 屈服了，还是用Scipy的吧
+        super(CubicSpline, self).__init__(x, y)
+
+    def _calc_coef(self):
+        self._sp_csp = scipy.interpolate.CubicSpline(self.x, self.y)
+
+    def interpolate(self, x, order):
+        if np.array(x).size == 0:
+            return np.empty_like(x)
+
+        val = self._sp_csp(x, order)
+        return val
+
+    def extrapolate(self, x, order):
+        '''
+        在超出已知数据范围的点上进行插值/外推，默认保持二阶导为0然后外推，即保持起点或终点的斜率
+        please notice that, if any x in the valid_x_range, the corresponding y maintains 0 value
+        '''
+        val = np.zeros_like(x)
+        if val.size == 0:
+            return val
+
+        x_0, x_end = self.valid_x_range
+        if order == 0:
+            y_prime_0 = self.evaluate(x_0, order=1)
+            y_prime_end = self.evaluate(x_end, order=1)
+            y_0 = self.evaluate(x_0, order=0)
+            y_end = self.evaluate(x_end, order=0)
+            val[x > x_end] = y_end + y_prime_end * (x[x>x_end] - x_end)
+            val[x < x_0] = y_0 + y_prime_0 * (x[x < x_end] - x_0)
+        elif order == 1:
+            y_prime_0 = self.evaluate(x_0, order=1)
+            y_prime_end = self.evaluate(x_end, order=1)
+            val[x > x_end] = y_prime_end
+            val[x < x_0] = y_prime_0
+        else:
+            pass  # because the higher order derivative is 0 anyway.
+
+        return val
+
+class myCubicSpline(InterpolationCurve):
     """
     Cubic CubicSpline class
     分段三次函数
@@ -582,7 +623,7 @@ class CubicSpline(InterpolationCurve):
     def __init__(self, x=None, y=None):
         self.b, self.c, self.d, self.w = [], [], [], []
         # 一定要把需要在calc_coef里面计算用到的属性定义在前面，因为下一句可能会调用calc_coef
-        super(CubicSpline, self).__init__(x, y)
+        super(myCubicSpline, self).__init__(x, y)
 
 
     def _calc_coef(self):
@@ -654,15 +695,25 @@ class CubicSpline(InterpolationCurve):
         i = self._search_index(x)
         dx = x - self.x[i]
         if order == 0:
-            val = self.a[i] + self.b[i] * dx + self.c[i] * dx ** 2.0 + self.d[i] * dx ** 3.0
+            val = np.polyval([self.d[i], self.c[i],self.b[i], self.a[i]], dx)
         elif order == 1:
-            val = self.b[i] + 2.0 * self.c[i] * dx + 3.0 * self.d[i] * dx ** 2.0
+            val = np.polyval([3*self.d[i], 2*self.c[i], self.b[i]], dx)
         elif order == 2:
-            val = 2.0 * self.c[i] + 6.0 * self.d[i] * dx
+            val = np.polyval([6 * self.d[i], 2 * self.c[i]], dx)
         elif order == 3:
             val = 6.0 * self.d[i]
         else:
             val = 0.
+        # if order == 0:
+        #     val = self.a[i] + self.b[i] * dx + self.c[i] * dx ** 2.0 + self.d[i] * dx ** 3.0
+        # elif order == 1:
+        #     val = self.b[i] + 2.0 * self.c[i] * dx + 3.0 * self.d[i] * dx ** 2.0
+        # elif order == 2:
+        #     val = 2.0 * self.c[i] + 6.0 * self.d[i] * dx
+        # elif order == 3:
+        #     val = 6.0 * self.d[i]
+        # else:
+        #     val = 0.
         return val
 
     def extrapolate(self, x, order):
