@@ -2,9 +2,10 @@ import math
 from typing import List
 import time
 import numpy as np
+import warnings
 
 import spider
-from spider.planner_zoo.BasicPlanner import BasicPlanner
+from spider.planner_zoo.BasePlanner import BasePlanner
 
 from spider.elements.map import RoutedLocalMap
 from spider.elements.trajectory import FrenetTrajectory
@@ -21,7 +22,7 @@ from spider.utils.collision import BoxCollisionChecker
 from spider.constraints import CartConstriantChecker
 
 
-class LatticePlanner(BasicPlanner):
+class LatticePlanner(BasePlanner):
     def __init__(self, config=None):
         super(LatticePlanner, self).__init__()
 
@@ -99,6 +100,7 @@ class LatticePlanner(BasicPlanner):
 
     def constraint_check(self, sorted_candidate_trajectories:List[FrenetTrajectory], sorted_cost, obstacles:TrackingBoxList):
         for traj, cost in zip(sorted_candidate_trajectories, sorted_cost):
+            traj = self.coordinate_transformer.frenet2cart4traj(traj, order=2) # 在笛卡尔坐标系下做碰撞检测
             if self.constraint_checker.check(traj, obstacles):
                 return traj, cost
 
@@ -167,7 +169,11 @@ class LatticePlanner(BasicPlanner):
         candidate_trajectories = self.trajectory_combiner.combine(lat_samples, long_samples)
 
         # 轨迹坐标转换，把每个轨迹点转到笛卡尔坐标
-        candidate_trajectories = [self.coordinate_transformer.frenet2cart4traj(t, order=2) for t in candidate_trajectories]
+        # todo: qzl:这一步耗时非常严重，有2个建议：
+        #  1. 评估暂时用不到笛卡尔坐标，碰撞检测目前在笛卡尔坐标下，所以可以在碰撞检测的时候再转换，不用提前把所有的都做坐标转换
+        #  2. 把碰撞检测直接整个放在Frenet坐标下，即先把障碍物及其预测轨迹变换到Frenet坐标，这样子所有的环节都在Frenet坐标进行，最后输出前再转换到笛卡尔
+        #  p.s.第一点的补充：坐标转换可以暂时先存储成函数或生成器，暂时先不执行，在碰撞检测的时候才执行
+        # candidate_trajectories = [self.coordinate_transformer.frenet2cart4traj(t, order=2) for t in candidate_trajectories]
 
         # 评估+筛选
         sorted_candidates, sorted_cost = self.trajectory_evaluator.evaluate_candidates(candidate_trajectories)
@@ -179,7 +185,7 @@ class LatticePlanner(BasicPlanner):
             print("Optimal trajectory found! s_dot_end=%.2f,l_end=%.2f" %
                   (optimal_trajectory.s_dot[-1], optimal_trajectory.l[-1]))
         else:
-            print("WARNING: NO feasible trajectory!")
+            warnings.warn("WARNING: NO feasible trajectory!")
 
         t2 = time.time()
         print("Planning Succeed! Time: %.2f seconds, FPS: %.2f" % (t2 - t1, 1 / (t2 - t1)))
@@ -267,7 +273,7 @@ if __name__ == '__main__':
                 fig = plt.gcf()
                 fig.canvas.draw()
                 image = np.frombuffer(fig.canvas.tostring_rgb(), dtype=np.uint8)
-                image = image.reshape(np.append(np.flip(fig.get_size_inches()) * fig.dpi, 3).astype(np.int))
+                image = image.reshape(np.append(np.flip(fig.get_size_inches()) * fig.dpi, 3).astype(int))
                 if video_writer is None:
                     video_writer = cv2.VideoWriter(video_name, fourcc, frame_rate, (image.shape[1], image.shape[0]))
                 video_writer.write(image)
@@ -418,4 +424,5 @@ if __name__ == '__main__':
             print("video_writer.release()")
 
 
-    intersection()
+    # intersection()
+    multilane()
